@@ -18,6 +18,11 @@
 #' Many of the measures in \pkg{mlr3} are implemented in \CRANpkg{mlr3measures} as ordinary functions.
 #'
 #' A guide on how to extend \CRANpkg{mlr3} with custom measures can be found in the [mlr3book](https://mlr3book.mlr-org.com).
+#' @section Inheriting:
+#' For some measures (such as confidence intervals from `mlr3inferr`) it is necessary that a measure
+#' returns more than one value.
+#' In such cases it is necessary to overwrite the public methods `$aggregate()` and/or `$score()` to return a named `numeric()`
+#' where at least one of its names corresponds to the `id` of the measure itself.
 #'
 #' @template param_id
 #' @template param_param_set
@@ -33,6 +38,12 @@
 #' @template param_packages
 #' @template param_label
 #' @template param_man
+#' @param obs_loss (`function` or `NULL`)\cr
+#'   The observation-wise loss function, e.g. [zero-one][mlr3measures::zero_one] for classification error.
+#' @param trafo (`list()` or `NULL`)\cr
+#'   An optional list with two elements, containing the transformation `"fn"` and its derivative `"deriv"`.
+#'   The transformation function is the function that is applied after aggregating the pointwise losses, i.e.
+#'   this requires an `$obs_loss` to be present. An example is `sqrt` for RMSE.
 #'
 #' @template seealso_measure
 #' @export
@@ -50,13 +61,21 @@ Measure = R6Class("Measure",
     #' @template field_param_set
     param_set = NULL,
 
+    #' @field obs_loss (`function()` | `NULL`)
+    #' Function to calculate the observation-wise loss.
+    obs_loss = NULL,
+
+    #' @field trafo (`list()` | `NULL`)
+    #' `NULL` or a list with two elements:
+    #' * `trafo`: the transformation function applied after aggregating
+    #'   observation-wise losses (e.g. `sqrt` for RMSE)
+    #' * `deriv`: The derivative of the `trafo`.
+    trafo = NULL,
+
     #' @field predict_type (`character(1)`)\cr
     #' Required predict type of the [Learner].
     predict_type = NULL,
 
-    #' @template field_predict_sets
-    predict_sets = NULL,
-
     #' @field check_prerequisites (`character(1)`)\cr
     #' How to proceed if one of the following prerequisites is not met:
     #'
@@ -94,10 +113,11 @@ Measure = R6Class("Measure",
     #'
     #' Note that this object is typically constructed via a derived classes, e.g. [MeasureClassif] or [MeasureRegr].
     initialize = function(id, task_type = NA, param_set = ps(), range = c(-Inf, Inf), minimize = NA, average = "macro",
-      aggregator = NULL, properties = character(), predict_type = "response",
+      aggregator = NULL, obs_loss = NULL, properties = character(), predict_type = "response",
       predict_sets = "test", task_properties = character(), packages = character(),
-      label = NA_character_, man = NA_character_) {
+      label = NA_character_, man = NA_character_, trafo = NULL) {
 
+      self$properties = unique(properties)
       self$id = assert_string(id, min.chars = 1L)
       self$label = assert_string(label, na.ok = TRUE)
       self$task_type = task_type
@@ -106,6 +126,11 @@ Measure = R6Class("Measure",
       self$minimize = assert_flag(minimize, na.ok = TRUE)
       self$average = average
       private$.aggregator = assert_function(aggregator, null.ok = TRUE)
+      self$obs_loss = assert_function(obs_loss, null.ok = TRUE)
+      self$trafo = assert_list(trafo, len = 2L, types = "function", null.ok = TRUE)
+      if (!is.null(self$trafo)) {
+        assert_permutation(names(trafo), c("fn", "deriv"))
+      }
 
       if (!is_scalar_na(task_type)) {
         assert_choice(task_type, mlr_reflections$task_types$type)
@@ -115,9 +140,8 @@ Measure = R6Class("Measure",
         assert_subset(task_properties, mlr_reflections$task_properties[[task_type]])
       }
 
-      self$properties = unique(properties)
       self$predict_type = predict_type
-      self$predict_sets = assert_subset(predict_sets, mlr_reflections$predict_sets, empty.ok = FALSE)
+      self$predict_sets = predict_sets
       self$task_properties = task_properties
       self$packages = union("mlr3", assert_character(packages, any.missing = FALSE, min.chars = 1L))
       self$man = assert_string(man, na.ok = TRUE)
@@ -171,8 +195,7 @@ Measure = R6Class("Measure",
     #' @return `numeric(1)`.
     score = function(prediction, task = NULL, learner = NULL, train_set = NULL) {
       assert_measure(self, task = task, learner = learner)
-      assert_prediction(prediction)
-
+      assert_prediction(prediction, null.ok = "requires_no_prediction" %nin% self$properties)
 
       if ("requires_task" %in% self$properties && is.null(task)) {
         stopf("Measure '%s' requires a task", self$id)
@@ -212,21 +235,44 @@ Measure = R6Class("Measure",
       switch(self$average,
         "macro" = {
           aggregator = self$aggregator %??% mean
-          tab = score_measures(rr, list(self), reassemble = FALSE, view = get_private(rr)$.view)
+          tab = score_measures(rr, list(self), reassemble = FALSE, view = get_private(rr)$.view,
+            iters = get_private(rr$resampling)$.primary_iters)
           set_names(aggregator(tab[[self$id]]), self$id)
         },
-        "micro" = self$score(rr$prediction(self$predict_sets), task = rr$task, learner = rr$learner),
-        "custom" = private$.aggregator(rr)
+        "micro" = {
+          if (is.null(get_private(rr$resampling)$.primary_iters)) {
+            self$score(rr$prediction(self$predict_sets), task = rr$task, learner = rr$learner)
+          } else {
+            prediction = do.call(c, rr$predictions(self$predict_sets)[get_private(rr$resampling)$.primary_iters])
+            self$score(prediction, task = rr$task, learner = rr$learner)
+          }
+        },
+        "custom" =  {
+          if (!is.null(get_private(rr$resampling)$.primary_iters) && "primary_iters" %nin% self$properties &&
+            !test_permutation(get_private(rr$resampling)$.primary_iters, seq_len(rr$resampling$iters))) {
+            stopf("Resample result has non-NULL primary_iters, but measure '%s' cannot handle them", self$id)
+          }
+          private$.aggregator(rr)
+        }
       )
     }
   ),
 
   active = list(
+    #' @template field_predict_sets
+    predict_sets = function(rhs) {
+      if (!missing(rhs)) {
+        private$.predict_sets = assert_subset(rhs, mlr_reflections$predict_sets, empty.ok = "requires_no_prediction" %in% self$properties)
+      }
+      private$.predict_sets
+    },
+
     #' @template field_hash
     hash = function(rhs) {
       assert_ro_binding(rhs)
-      calculate_hash(class(self), self$id, self$param_set$values, private$.score, private$.average,
-        private$.aggregator, self$predict_sets, mget(private$.extra_hash, envir = self))
+      calculate_hash(class(self), self$id, self$param_set$values, private$.score,
+        private$.average, private$.aggregator, self$obs_loss, self$trafo,
+        self$predict_sets, mget(private$.extra_hash, envir = self))
     },
 
     #' @field average (`character(1)`)\cr
@@ -261,6 +307,7 @@ Measure = R6Class("Measure",
   ),
 
   private = list(
+    .predict_sets = NULL,
     .extra_hash = character(),
     .average = NULL,
     .aggregator = NULL
@@ -284,8 +331,11 @@ Measure = R6Class("Measure",
 #' @return (`numeric()`).
 #' @noRd
 score_single_measure = function(measure, task, learner, train_set, prediction) {
-  if (is.null(prediction)) {
-    return(NaN)
+  if (!length(measure$predict_sets)) {
+    score = get_private(measure)$.score(
+      prediction = NULL, task = task, learner = learner, train_set = train_set
+    )
+    return(score)
   }
 
   # merge multiple predictions (on different predict sets) to a single one
@@ -301,6 +351,12 @@ score_single_measure = function(measure, task, learner, train_set, prediction) {
   # convert pdata to regular prediction
   prediction = as_prediction(prediction, check = FALSE)
 
+  if (is.null(prediction) && length(measure$predict_sets)) {
+    return(NaN)
+  }
+
+
+
   if (!is_scalar_na(measure$predict_type) && measure$predict_type %nin% prediction$predict_types) {
     # TODO lgr$debug()
     return(NaN)
@@ -325,15 +381,21 @@ score_single_measure = function(measure, task, learner, train_set, prediction) {
 #'
 #' @param obj ([ResampleResult] | [BenchmarkResult]).
 #' @param measures (list of [Measure]).
+#' @param iters (`integer()` or `NULL`)
+#' To which iterations of the resample result to restrict the scoring.
 #'
 #' @return (`data.table()`) with added score columns.
 #'
 #' @noRd
-score_measures = function(obj, measures, reassemble = TRUE, view = NULL) {
+score_measures = function(obj, measures, reassemble = TRUE, view = NULL, iters = NULL) {
   reassemble_learners = reassemble ||
     some(measures, function(m) any(c("requires_learner", "requires_model") %in% m$properties))
   tab = get_private(obj)$.data$as_data_table(view = view, reassemble_learners = reassemble_learners, convert_predictions = FALSE)
 
+  if (!is.null(iters)) {
+    tab = tab[list(iters), on = "iteration"]
+  }
+
   tmp = unique(tab, by = c("task_hash", "learner_hash"))[, c("task", "learner"), with = FALSE]
 
   for (measure in measures) {
