@@ -27,7 +27,8 @@ MeasureAIC = R6Class("MeasureAIC",
         id = "aic",
         task_type = NA_character_,
         param_set = param_set,
-        properties = c("na_score", "requires_learner", "requires_model"),
+        predict_sets = NULL,
+        properties = c("na_score", "requires_learner", "requires_model", "requires_no_prediction"),
         predict_type = NA_character_,
         minimize = TRUE,
         label = "Akaike Information Criterion",
