@@ -25,7 +25,8 @@ MeasureBIC = R6Class("MeasureBIC",
       super$initialize(
         id = "bic",
         task_type = NA_character_,
-        properties = c("na_score", "requires_learner", "requires_model"),
+        properties = c("na_score", "requires_learner", "requires_model", "requires_no_prediction"),
+        predict_sets = NULL,
         predict_type = NA_character_,
         minimize = TRUE,
         label = "Bayesian Information Criterion",
