@@ -22,7 +22,8 @@ MeasureOOBError = R6Class("MeasureOOBError",
       super$initialize(
         id = "oob_error",
         task_type = NA_character_,
-        properties = c("na_score", "requires_learner"),
+        properties = c("na_score", "requires_learner", "requires_no_prediction"),
+        predict_sets = NULL,
         predict_type = NA_character_,
         range = c(-Inf, Inf),
         minimize = TRUE,
