@@ -95,6 +95,21 @@ Prediction = R6Class("Prediction",
       set_names(scores, ids(measures))
     },
 
+    #' @description
+    #' Calculates the observation-wise loss via the loss function set in the
+    #' [Measure]'s field `obs_loss`.
+    #' Returns a `data.table()` with the columns `row_ids`, `truth`, `response` and
+    #' one additional numeric column for each measure, named with the respective measure id.
+    #' If there is no observation-wise loss function for the measure, the column is filled with
+    #' `NA` values.
+    #' Note that some measures such as RMSE, do have an `$obs_loss`, but they require an
+    #' additional transformation after aggregation, in this example taking the square-root.
+    obs_loss = function(measures = NULL) {
+      measures = as_measures(measures, task_type = self$task_type)
+      get_obs_loss(as.data.table(self), measures)
+    },
+
+
     #' @description
     #' Filters the [Prediction], keeping only predictions for the provided row_ids.
     #' This changes the object in-place, you need to create a clone to preserve
