@@ -111,6 +111,8 @@ assert_learners = function(learners, task = NULL, task_type = NULL, properties =
   invisible(lapply(learners, assert_learner, task = task, task_type = NULL, properties = properties, .var.name = .var.name))
 }
 
+# this does not check the validation task, as this is only possible once the validation set is known,
+# which happens during worker(), so it cannot be checked before that
 assert_task_learner = function(task, learner, cols = NULL) {
   pars = learner$param_set$get_values(type = "only_token", check_required = FALSE)
   if (length(pars) > 0) {
@@ -145,6 +147,11 @@ assert_task_learner = function(task, learner, cols = NULL) {
         task$id, tmp[1L], learner$id)
     }
   }
+
+  validate = get0("validate", learner)
+  if (!is.null(task$internal_valid_task) && (is.numeric(validate) || identical(validate, "test"))) {
+    stopf("Parameter 'validate' of Learner '%s' cannot be set to 'test' or a ratio when internal_valid_task is present, remove it first", learner$id)
+  }
 }
 
 #' @export
@@ -159,6 +166,27 @@ assert_learnable = function(task, learner) {
 #' @export
 #' @rdname mlr_assertions
 assert_predictable = function(task, learner) {
+  if (!is.null(learner$state$train_task)) {
+    train_task = learner$state$train_task
+    cols_train = train_task$feature_names
+    cols_predict = task$feature_names
+
+    if (!test_permutation(cols_train, cols_predict)) {
+      stopf("Learner '%s' has received tasks with different columns in train and predict.", learner$id)
+    }
+
+    ids = train_task$col_info[get("id") %in% cols_train, "id"]$id
+    ci_predict = task$col_info[list(ids), c("id", "type", "levels"), on = "id"]
+    ci_train = train_task$col_info[list(ids), c("id", "type", "levels"), on = "id"]
+
+    ok = all(ci_train$type == ci_predict$type) &&
+      all(pmap_lgl(list(x = ci_train$levels, y = ci_predict$levels), identical))
+
+    if (!ok) {
+      lg$warn("Learner '%s' received task with different column info (feature type or level ordering) during train and predict.", learner$id)
+    }
+  }
+
   assert_task_learner(task, learner, cols = task$feature_names)
 }
 
@@ -253,7 +281,8 @@ assert_resamplings = function(resamplings, instantiated = NULL, .var.name = vnam
 #' @export
 #' @param prediction ([Prediction]).
 #' @rdname mlr_assertions
-assert_prediction = function(prediction, .var.name = vname(prediction)) {
+assert_prediction = function(prediction, .var.name = vname(prediction), null.ok = FALSE) {
+  if (null.ok && is.null(prediction)) return(prediction)
   assert_class(prediction, "Prediction", .var.name = .var.name)
 }
 
