@@ -5,12 +5,12 @@ fix_factor_levels = function(data, levels, ...) {
 #' @export
 fix_factor_levels.data.table = function(data, levels, ...) { # nolint
   levels = levels[intersect(names(levels), names(data))]
-  imap(levels, function(lvls, id) {
+  iwalk(levels, function(lvls, id, data) {
     x = data[[id]]
     if (!identical(levels(x), lvls)) {
       set(data, j = id, value = factor(x, levels = lvls, ordered = is.ordered(x)))
     }
-  })
+  }, data = data)
   data[]
 }
 
